@@ -1,3 +1,5 @@
+/* $Id: acconfig.h,v 1.5 2000/08/07 02:16:24 astyanax Exp $ */
+
 /* Define this if you have the wresize function in your ncurses-type library */
 #undef HAVE_WRESIZE
 
@@ -22,5 +24,3 @@
 /* Define to use the slang wrappers for curses instead of native curses */
 #undef USE_SLANG
 
-/* Define this if you have a behaving TABSIZE global in your curses lib */
-#undef HAVE_TABSIZE
