@@ -1,3 +1,4 @@
+/* $Id: cut.c,v 1.19 2000/08/07 14:58:26 astyanax Exp $ */
 /**************************************************************************
  *   cut.c                                                                *
  *                                                                        *
@@ -97,8 +98,7 @@ void cut_marked_segment(filestruct * top, int top_x, filestruct * bot,
     free(top->data);
     top->data = tmpstr;
 
-    bot->data[bot_x] = 0;
-    align(&bot->data);
+    null_at(bot->data, bot_x);
     next = bot->next;
 
     /* We explicitly don't decrement totlines here because we don't snarf
@@ -123,8 +123,8 @@ void cut_marked_segment(filestruct * top, int top_x, filestruct * bot,
 	    filebot = top;
 	}
     }
-    if (top->lineno < edittop->lineno)
-	edit_update(top);
+     if (top->lineno < edittop->lineno)
+	edit_update(top, CENTER);
 }
 #endif
 
@@ -133,22 +133,47 @@ int do_cut_text(void)
     filestruct *tmp, *fileptr = current;
 #ifndef NANO_SMALL
     char *tmpstr;
-    int newsize;
+    int newsize, cuttingtoend = 0;
 #endif
 
+    check_statblank();
     if (fileptr == NULL || fileptr->data == NULL)
 	return 0;
 
     tmp = fileptr->next;
 
-    if (!ISSET(KEEP_CUTBUFFER) || ISSET(MARK_ISSET)) {
+    if (!ISSET(KEEP_CUTBUFFER)) {
 	free_filestruct(cutbuffer);
 	cutbuffer = NULL;
+
 #ifdef DEBUG
 	fprintf(stderr, _("Blew away cutbuffer =)\n"));
 #endif
     }
+
+    /* Must let cutbuffer get blown away first before we do this... */
+    if (fileptr == filebot)	
+	return 0;
+
 #ifndef NANO_SMALL
+    if (ISSET(CUT_TO_END) && !ISSET(MARK_ISSET)) {
+	if (current_x == strlen(current->data))
+	{
+	    do_delete();
+	    SET(KEEP_CUTBUFFER);
+	    marked_cut = 2;
+	    return 1;
+	}
+	else
+	{
+	    SET(MARK_ISSET);
+	    SET(KEEP_CUTBUFFER);
+
+	    mark_beginx = strlen(current->data);
+	    mark_beginbuf = current;
+	    cuttingtoend = 1;
+	}
+    }
     if (ISSET(MARK_ISSET)) {
 	if (current->lineno == mark_beginbuf->lineno) {
 	    tmp = copy_node(current);
@@ -183,9 +208,14 @@ int do_cut_text(void)
 
 	placewewant = xplustabs();
 	UNSET(MARK_ISSET);
+
 	marked_cut = 1;
 	set_modified();
-	edit_update(current);
+	if (cuttingtoend)
+	    edit_refresh();
+	else
+	    edit_update(current, CENTER);
+
 	return 1;
 #else
     if (0) {
@@ -197,10 +227,11 @@ int do_cut_text(void)
 	    tmp = fileptr;
 	    fileage = fileptr;
 	    add_to_cutbuffer(fileptr->prev);
+	    totsize--; /* get the newline */
 	    totlines--;
 	    fileptr->prev = NULL;
-	    edit_update(fileage);
 	    current = fileptr;
+	    edit_update(fileage, CENTER);
 	} else {
 	    add_to_cutbuffer(fileptr);
 	    fileage = make_new_node(NULL);
@@ -216,18 +247,9 @@ int do_cut_text(void)
 	    (fileptr->next)->prev = fileptr->prev;
 	    current = fileptr->next;
 	    totlines--;
-	} else {
-	    /* we're deleteing the last line
-	       and replacing it with a dummy line,
-	       so totlines is the same */
-	    tmp = fileptr->prev;
-	    tmp->next = make_new_node(tmp);
-	    tmp = tmp->next;
-	    tmp->data = nmalloc(1);
-	    strcpy(tmp->data, "");
-	    current = tmp;
-	    filebot = tmp;
-	}
+	    totsize--; /* get the newline */
+	}	/* No longer an else here, because we never get here anymore...
+		   No need to cut the magic line, as it's empty */
 
 	add_to_cutbuffer(fileptr);
     }
@@ -259,6 +281,7 @@ int do_uncut_text(void)
     int i;
 
     wrap_reset();
+    check_statblank();
     if (cutbuffer == NULL || fileptr == NULL)
 	return 0;		/* AIEEEEEEEEEEEE */
 
@@ -270,7 +293,7 @@ int do_uncut_text(void)
 
     /* Hook newbuf into fileptr */
 #ifndef NANO_SMALL
-    if (marked_cut == 1) {
+    if (marked_cut) {
 	/* If there's only one line in the cutbuffer */
 	if (cutbuffer->next == NULL) {
 	    tmpstr =
@@ -286,7 +309,6 @@ int do_uncut_text(void)
 
 	    placewewant = xplustabs();
 	    update_cursor();
-	    renumber(current);
 	} else {		/* yuck -- no kidding! */
 	    tmp = current->next;
 	    /* New beginning */
@@ -329,13 +351,27 @@ int do_uncut_text(void)
 
 	    i = editbot->lineno;
 
-	    renumber(current);
-
 	    current = newend;
 	    if (i <= newend->lineno)
-		edit_update(current);
+		edit_update(current, CENTER);
 	}
 
+	/* If marked cut == 2, that means that we're doing a cut to end
+	   and we don't want anything else on the line, so we have to
+	   screw up all the work we just did and separate the line.  There
+	   must be a better way to do this, but not at 1AM on a work night. */
+
+	if (marked_cut == 2 && current_x != strlen(current->data)) {
+	    tmp = make_new_node(current);
+	    tmp->data = nmalloc(strlen(&current->data[current_x]) + 1);
+	    strcpy(tmp->data, &current->data[current_x]);
+	    splice_node(current, tmp, current->next);
+	    null_at(current->data, current_x);
+	    current = current->next;
+	    current_x = 0;
+	    placewewant = 0;
+	}
+	renumber(current->prev);
 	dump_buffer(fileage);
 	dump_buffer(cutbuffer);
 	set_modified();
@@ -368,8 +404,8 @@ int do_uncut_text(void)
 
     i = editbot->lineno;
     renumber(newbuf);
-    if (i < newend->lineno)
-	edit_update(fileptr);
+     if (i < newend->lineno)
+	edit_update(fileptr, CENTER);
 
     dump_buffer_reverse(fileptr);
 
