@@ -1,3 +1,4 @@
+/* $Id: global.c,v 1.6 2000/08/06 21:13:45 astyanax Exp $ */
 /**************************************************************************
  *   global.c                                                             *
  *                                                                        *
@@ -60,6 +61,8 @@ int placewewant = 0;		/* The collum we'd like the cursor
 				   to jump to when we go to the
 				   next or previous line */
 
+int tabsize = 8;		/* Our internal tabsize variable */
+
 char *hblank;			/* A horizontal blank line */
 char *help_text;		/* The text in the help window */
 
@@ -76,6 +79,12 @@ shortcut writefile_list[WRITEFILE_LIST_LEN];
 shortcut help_list[HELP_LIST_LEN];
 shortcut spell_list[SPELL_LIST_LEN];
 
+/* Regular expressions */
+
+regex_t search_regexp;          /* Global to store compiled search regexp */
+regmatch_t regmatches[10];      /* Match positions for parenthetical
+                                   subexpressions, max of 10 */ 
+
 /* Initialize a struct *without* our lovely braces =( */
 void sc_init_one(shortcut * s, int key, char *desc, char *help, int alt,
 		 int misc1, int misc2, int view, int (*func) (void))
@@ -267,10 +276,13 @@ void shortcut_init(void)
 		nano_case_msg, 0, 0, 0, VIEW, 0);
 
 
-    sc_init_one(&whereis_list[3], NANO_OTHERSEARCH_KEY, _("To Replace"),
+    sc_init_one(&whereis_list[3], NANO_OTHERSEARCH_KEY, _("Replace"),
 		nano_replace_msg, 0, 0, 0, VIEW, do_replace);
 
-    sc_init_one(&whereis_list[4], NANO_CANCEL_KEY, _("Cancel"),
+    sc_init_one(&whereis_list[4], NANO_FROMSEARCHTOGOTO_KEY, _("Goto Line"),
+		nano_goto_msg, 0, 0, 0, VIEW, do_gotoline_void);
+
+    sc_init_one(&whereis_list[5], NANO_CANCEL_KEY, _("Cancel"),
 		nano_cancel_msg, 0, 0, 0, VIEW, 0);
 
 
@@ -283,10 +295,13 @@ void shortcut_init(void)
     sc_init_one(&replace_list[2], NANO_CASE_KEY, _("Case Sens"),
 		nano_case_msg, 0, 0, 0, VIEW, 0);
 
-    sc_init_one(&replace_list[3], NANO_OTHERSEARCH_KEY, _("To Search"),
+    sc_init_one(&replace_list[3], NANO_OTHERSEARCH_KEY, _("No Replace"),
 		nano_whereis_msg, 0, 0, 0, VIEW, do_search);
 
-    sc_init_one(&replace_list[4], NANO_CANCEL_KEY, _("Cancel"),
+    sc_init_one(&replace_list[4], NANO_FROMSEARCHTOGOTO_KEY, _("Goto Line"),
+		nano_goto_msg, 0, 0, 0, VIEW, do_gotoline_void);
+
+    sc_init_one(&replace_list[5], NANO_CANCEL_KEY, _("Cancel"),
 		nano_cancel_msg, 0, 0, 0, VIEW, 0);
 
 
