@@ -1,3 +1,4 @@
+/* $Id: nano.c,v 1.45 2000/08/06 21:09:27 astyanax Exp $ */
 /**************************************************************************
  *   nano.c                                                               *
  *                                                                        *
@@ -58,13 +59,11 @@
 #endif
 
 /* Former globals, now static */
-char *last_search;		/* Last string we searched for */
-char *last_replace;		/* Last replacement string */
-int temp_opt = 0;		/* Editing temp file (pico -t option) */
+char *last_search = "\0";		/* Last string we searched for */
+char *last_replace = "\0";		/* Last replacement string */
 int fill = 0;			/* Fill - where to wrap lines, basically */
 static char *alt_speller;	/* Alternative spell command */
 struct termios oldterm;		/* The user's original term settings */
-static char *alt_speller;	/* Alternative spell command */
 static char *help_text_init = "";
 				/* Initial message, not including shortcuts */
 
@@ -96,10 +95,15 @@ void die(char *msg, ...)
     va_end(ap);
 
     /* if we can't save we have REAL bad problems,
-     * but we might as well TRY.  FIXME: This should probabally base it
-     * off of the current filename */
-    write_file("nano.save", 0);
-
+     * but we might as well TRY. */
+    if(filename[0] == '\0') {
+	write_file("nano.save", 0);
+    } else {
+	char buf[BUFSIZ];
+	strncpy(buf,filename,BUFSIZ);
+	strncat(buf,".save",BUFSIZ - strlen(buf));
+	write_file(buf, 0);
+    }
     /* Restore the old term settings */
     tcsetattr(0, TCSANOW, &oldterm);
 
@@ -304,14 +308,23 @@ void align(char **strp)
     *strp = nrealloc(*strp, strlen(*strp) + 1);
 }
 
+/* Null a string at a certain index and align it */
+void null_at(char *data, int index)
+{
+    data[index] = 0;
+    align(&data);
+}
+
 void usage(void)
 {
 #ifdef HAVE_GETOPT_LONG
     printf(_("Usage: nano [GNU long option] [option] +LINE <file>\n\n"));
     printf(_("Option		Long option		Meaning\n"));
-#ifdef HAVE_TABSIZE
     printf(_
 	   (" -T 		--tabsize=[num]		Set width of a tab to num\n"));
+#ifdef _POSIX_VERSION
+    printf(_
+         (" -R		--regexp		Use regular expressions for search\n"));
 #endif
     printf
 	(_
@@ -320,6 +333,10 @@ void usage(void)
 	   (" -c 		--const			Constantly show cursor position\n"));
     printf(_
 	   (" -h 		--help			Show this message\n"));
+#ifndef NANO_SMALL
+    printf(_
+	   (" -k 		--cut			Let ^K cut from cursor to end of line\n"));
+#endif
     printf(_
 	   (" -i 		--autoindent		Automatically indent new lines\n"));
     printf(_
@@ -351,12 +368,14 @@ void usage(void)
 #else
     printf(_("Usage: nano [option] +LINE <file>\n\n"));
     printf(_("Option		Meaning\n"));
-#ifdef HAVE_TABSIZE
     printf(_(" -T [num]	Set width of a tab to num\n"));
-#endif
+    printf(_(" -R		Use regular expressions for search\n"));
     printf(_(" -V 		Print version information and exit\n"));
     printf(_(" -c 		Constantly show cursor position\n"));
     printf(_(" -h 		Show this message\n"));
+#ifndef NANO_SMALL
+    printf(_(" -k 		Let ^K cut from cursor to end of line\n"));
+#endif
     printf(_(" -i 		Automatically indent new lines\n"));
     printf(_
 	   (" -l 		Don't follow symbolic links, overwrite.\n"));
@@ -402,6 +421,16 @@ filestruct *make_new_node(filestruct * prevnode)
     return newnode;
 }
 
+/* Splice a node into an existing filestruct */
+void splice_node(filestruct *begin, filestruct *new, filestruct *end)
+{
+    new->next = end;
+    new->prev = begin;
+    begin->next = new;
+    if (end != NULL)
+	end->prev = new;
+}
+
 int do_mark()
 {
 #ifdef NANO_SMALL
@@ -441,6 +470,13 @@ void nano_small_msg(void)
 /* The user typed a printable character; add it to the edit buffer */
 void do_char(char ch)
 {
+    /* magic-line: when a character is inserted on the current magic line,
+     * it means we need a new one! */
+    if(filebot == current && current->data[0] == '\0') {
+	new_magicline();
+	fix_editbot();
+    }
+
     /* More dangerousness fun =) */
     current->data = nrealloc(current->data, strlen(current->data) + 2);
     memmove(&current->data[current_x + 1],
@@ -488,34 +524,35 @@ int do_enter(filestruct * inptr)
     }
     *tmp = 0;
 
-    new->next = inptr->next;
-    new->prev = inptr;
-    inptr->next = new;
-    if (new->next != NULL)
-	new->next->prev = new;
-    else {
+    if (inptr->next == NULL) {
 	filebot = new;
 	editbot = new;
     }
+    splice_node(inptr, new, inptr->next);
 
     totsize++;
     renumber(current);
     current = new;
     align(&current->data);
 
+    /* The logic here is as follows:
+     *    -> If we are at the bottom of the buffer, we want to recenter
+     *       (read: rebuild) the screen and forcably move the cursor.
+     *    -> otherwise, we want simply to redraw the screen and update
+     *       where we think the cursor is.
+     */
     if (current_y == editwinrows - 1) {
-	edit_update(current);
-
-	/* FIXME - figure out why the hell this is needed =) */
-	reset_cursor();
-    } else
+	edit_update(current, CENTER);
+	reset_cursor(); 
+    } else {
 	current_y++;
+	edit_refresh();
+	update_cursor();
+    }
 
     totlines++;
     set_modified();
 
-    update_cursor();
-    edit_refresh();
     placewewant = xplustabs();
     return 1;
 }
@@ -561,7 +598,7 @@ void do_next_word(void)
     current_x = i;
     placewewant = xplustabs();
     if (current->lineno >= editbot->lineno)
-	edit_update(current);
+	edit_update(current, CENTER);
 
 }
 
@@ -606,16 +643,14 @@ void do_wrap(filestruct * inptr, char input_char)
 	}
 
 	if (inptr->data[i] == NANO_CONTROL_I) {
-	    if (i_tabs % TABSIZE != 0);
-	    i_tabs += TABSIZE - (i_tabs % TABSIZE);
+	    if (i_tabs % tabsize != 0);
+	    i_tabs += tabsize - (i_tabs % tabsize);
 	}
 
 	if (current_word_end_t > fill)
 	    break;
     }
 
-    assert(current_word_end_t > fill);
-
     /* There are a few (ever changing) cases of what the line could look like.
      * 1) only one word on the line before wrap point.
      *    a) one word takes up the whole line with no starting spaces.
@@ -686,8 +721,7 @@ void do_wrap(filestruct * inptr, char input_char)
 	    down = 1;
 	}
 
-	inptr->data = nrealloc(inptr->data, current_x + 1);
-	inptr->data[current_x] = 0;
+	null_at(inptr->data, current_x);
 
 	if (ISSET(MARK_ISSET) && (mark_beginbuf == inptr)) {
 	    mark_beginbuf = temp;
@@ -734,9 +768,7 @@ void do_wrap(filestruct * inptr, char input_char)
 	    if (isspace(input_char) && (current_x == current_word_start)) {
 		current_x = current_word_start;
 
-		inptr->data =
-		    nrealloc(inptr->data, current_word_start + 1);
-		inptr->data[current_word_start] = 0;
+		null_at(inptr->data, current_word_start);
 	    } else {
 
 		while (isspace(inptr->data[i])) {
@@ -819,10 +851,12 @@ void do_wrap(filestruct * inptr, char input_char)
 
 
     totlines++;
-    totsize++;
+    /* Everything about it makes me want this line here but it causes
+     * totsize to be high by one for some reason.  Sigh. (Rob) */
+    /* totsize++; */
 
     renumber(inptr);
-    edit_update_top(edittop);
+    edit_update(edittop, TOP);
 
 
     /* Move the cursor to the new line if appropriate. */
@@ -835,7 +869,7 @@ void do_wrap(filestruct * inptr, char input_char)
 	do_right();
     }
 
-    edit_update_top(edittop);
+    edit_update(edittop, TOP);
     reset_cursor();
     edit_refresh();
 }
@@ -887,33 +921,6 @@ void do_early_abort(void)
     blank_statusbar_refresh();
 }
 
-int page_up(void)
-{
-    wrap_reset();
-    current_x = 0;
-    placewewant = 0;
-
-    if (current == fileage)
-	return 0;
-
-    current_y = 0;
-    edit_update_bot(edittop);
-    update_cursor();
-
-    UNSET(KEEP_CUTBUFFER);
-    check_statblank();
-    return 1;
-}
-
-void delete_buffer(filestruct * inptr)
-{
-    if (inptr != NULL) {
-	delete_buffer(inptr->next);
-	free(inptr->data);
-	free(inptr);
-    }
-}
-
 int do_backspace(void)
 {
     filestruct *previous, *tmp;
@@ -946,7 +953,7 @@ int do_backspace(void)
 		current = previous->next;
 	    else
 		current = previous;
-	    page_up();
+	    page_up_center();
 	} else {
 	    if (previous->next)
 		current = previous->next;
@@ -959,13 +966,22 @@ int do_backspace(void)
 	if (tmp == filebot) {
 	    filebot = current;
 	    editbot = current;
+
+	    /* Recreate the magic line if we're deleting it AND if the
+		line we're on now is NOT blank.  if it is blank we
+		can just use IT for the magic line.   This is how Pico
+		appears to do it, in any case */
+	    if (strcmp(current->data, "")) {
+		new_magicline();
+		fix_editbot();
+		totsize++;
+	    }
 	}
 
 	current = previous;
 	renumber(current);
 	previous_line();
 	totlines--;
-
 #ifdef DEBUG
 	fprintf(stderr, _("After, data = \"%s\"\n"), current->data);
 #endif
@@ -1006,9 +1022,16 @@ int do_delete(void)
 	delete_node(foo);
 	update_line(current, current_x);
 
+	/* Please see the comment in do_basckspace if you don't understand
+	   this test */
+	if (current == filebot && strcmp(current->data, ""))
+	{
+	    new_magicline();
+	    fix_editbot();
+	    totsize++;
+	}
 	renumber(current);
 	totlines--;
-
     } else
 	return 0;
 
@@ -1031,66 +1054,21 @@ void exit_spell(char *tmpfilename, char *foo)
 
     if (remove(tmpfilename) == -1)
 	statusbar(_("Error deleting tempfile, ack!"));
+    display_main_list();
 }
 
 /*
  * This is Chris' very ugly spell function.  Someone please make this
  * better =-)
  */
-int do_oldspell(void)
-{
-    char *temp, *foo;
-    int i;
-
-    if ((temp = tempnam(0, "nano.")) == NULL) {
-	statusbar(_("Could not create a temporary filename: %s"),
-		  strerror(errno));
-	return 0;
-    }
-    if (write_file(temp, 1) == -1)
-	return 0;
-
-    if (alt_speller) {
-	foo = nmalloc(strlen(temp) + strlen(alt_speller) + 2);
-	sprintf(foo, "%s %s", alt_speller, temp);
-    } else {
-
-	/* For now, we only try ispell because we're not capable of
-	   handling the normal spell program (yet...) */
-	foo = nmalloc(strlen(temp) + 8);
-	sprintf(foo, "ispell %s", temp);
-    }
-
-    endwin();
-    resetty();
-    if (alt_speller) {
-	if ((i = system(foo)) == -1 || i == 32512) {
-	    statusbar(_("Could not invoke spell program \"%s\""),
-		      alt_speller);
-	    exit_spell(temp, foo);
-	    return 0;
-	}
-    } else if ((i = system(foo)) == -1 || i == 32512) {	/* Why 32512? I dont know! */
-	statusbar(_("Could not invoke \"ispell\""));
-	exit_spell(temp, foo);
-	return 0;
-    }
-    initscr();
-
-    free_filestruct(fileage);
-    global_init();
-    open_file(temp, 0, 1);
-    edit_update(fileage);
-    set_modified();
-    exit_spell(temp, foo);
-    statusbar(_("Finished checking spelling"));
-    return 1;
-}
-
 int do_spell(void)
 {
+#ifdef NANO_SMALL
+    nano_small_msg();
+    return 1;
+#else
     char *temp, *foo;
-    int i;
+    int i, size;
 
     if ((temp = tempnam(0, "nano.")) == NULL) {
 	statusbar(_("Could not create a temporary filename: %s"),
@@ -1101,18 +1079,19 @@ int do_spell(void)
 	return 0;
 
     if (alt_speller) {
-	foo = nmalloc(strlen(temp) + strlen(alt_speller) + 2);
-	sprintf(foo, "%s %s", alt_speller, temp);
+	size = strlen(temp) + strlen(alt_speller) + 2;
+	foo = nmalloc(size);
+	snprintf(foo, size, "%s %s", alt_speller, temp);
     } else {
 
 	/* For now, we only try ispell because we're not capable of
 	   handling the normal spell program (yet...) */
-	foo = nmalloc(strlen(temp) + 8);
-	sprintf(foo, "ispell %s", temp);
+	size = strlen(temp) + 8;
+	foo = nmalloc(size);
+	snprintf(foo, size, "ispell %s", temp);
     }
 
     endwin();
-    resetty();
     if (alt_speller) {
 	if ((i = system(foo)) == -1 || i == 32512) {
 	    statusbar(_("Could not invoke spell program \"%s\""),
@@ -1125,16 +1104,18 @@ int do_spell(void)
 	exit_spell(temp, foo);
 	return 0;
     }
-    initscr();
+/*    initscr(); */
+    refresh();
 
     free_filestruct(fileage);
     global_init();
     open_file(temp, 0, 1);
-    edit_update(fileage);
+    edit_update(fileage, CENTER);
     set_modified();
     exit_spell(temp, foo);
     statusbar(_("Finished checking spelling"));
     return 1;
+#endif
 }
 
 int do_exit(void)
@@ -1144,7 +1125,7 @@ int do_exit(void)
     if (!ISSET(MODIFIED))
 	finish(0);
 
-    if (temp_opt) {
+    if (ISSET(TEMP_OPT)) {
 	i = 1;
     } else {
 	i =
@@ -1287,14 +1268,10 @@ void handle_sigwinch(int s)
 #endif				/* HAVE_WRESIZE */
 #endif				/* HAVE_NCURSES_H */
 
-    editbot = edittop;
-
-    for (i = 0; (i <= editwinrows - 1) && (editbot->next != NULL)
-	 && (editbot->next != filebot); i++)
-	editbot = editbot->next;
+    fix_editbot();
 
     if (current_y > editwinrows - 1) {
-	edit_update(editbot);
+	edit_update(editbot, CENTER);
     }
     erase();
 
@@ -1422,56 +1399,61 @@ int do_justify(void)
 
 	unlink_node(tmpnode);
 	delete_node(tmpnode);
-
     }
 
+    totsize -= strlen(current->data);
+
     justify_format(current->data);
 
     slen = strlen(current->data);
-    while ((strlenpt(current->data) > (fill))
-	   && !no_spaces(current->data)) {
-	int i = 0;
-	int len2 = 0;
-	filestruct *tmpline = nmalloc(sizeof(filestruct));
+    totsize += slen;
 
-	/* Start at fill , unless line isn't that long (but it appears at least
-	 * fill long with tabs.
-	 */
-	if (slen > fill)
-	    i = fill;
-	else
-	    i = slen;
-	for (; i > 0; i--) {
-	    if (isspace(current->data[i]) &&
-		((strlenpt(current->data) - strlen(current->data + i)) <=
-		 fill)) break;
-	}
-	if (!i)
-	    break;
+    if((strlenpt(current->data) > (fill))
+           && !no_spaces(current->data)) {
+	do {
+	    int i = 0;
+	    int len2 = 0;
+	    filestruct *tmpline = nmalloc(sizeof(filestruct));
 
-	current->data[i] = '\0';
+	    /* Start at fill , unless line isn't that long (but it 
+	     * appears at least fill long with tabs.
+	     */
+	    if (slen > fill)
+		i = fill;
+	    else
+		i = slen;
+	    for (; i > 0; i--) {
+		if (isspace(current->data[i]) &&
+		    ((strlenpt(current->data) - strlen(current->data +i)) <=
+		     fill)) break;
+	    }
+	    if (!i)
+	        break;
 
-	len2 = strlen(current->data + i + 1);
-	tmpline->data = nmalloc(len2 + 1);
+	    current->data[i] = '\0';
 
-	/* Skip the white space in current. */
-	memcpy(tmpline->data, current->data + i + 1, len2);
-	tmpline->data[len2] = '\0';
+	    len2 = strlen(current->data + i + 1);
+	    tmpline->data = nmalloc(len2 + 1);
 
-	current->data = nrealloc(current->data, i + 1);
+	    /* Skip the white space in current. */
+	    memcpy(tmpline->data, current->data + i + 1, len2);
+	    tmpline->data[len2] = '\0';
 
-	tmpline->prev = current;
-	tmpline->next = current->next;
-	if (current->next != NULL)
-	    current->next->prev = tmpline;
+	    current->data = nrealloc(current->data, i + 1);
 
-	current->next = tmpline;
-	current = tmpline;
-	slen -= i + 1;
-	current_y++;
+	    tmpline->prev = current;
+	    tmpline->next = current->next;
+	    if (current->next != NULL)
+		current->next->prev = tmpline;
+
+	    current->next = tmpline;
+	    current = tmpline;
+	    slen -= i + 1;
+	    current_y++;
+        } while ((strlenpt(current->data) > (fill))
+		&& !no_spaces(current->data));
     }
 
-
     if (current->next)
 	current = current->next;
     else
@@ -1486,20 +1468,13 @@ int do_justify(void)
 
     if ((current_y < 0) || (current_y >= editwinrows - 1)
 	|| (initial_y <= 0)) {
-	edit_update(current);
+	edit_update(current, CENTER);
 	center_cursor();
     } else {
-	int i = 0;
-
-	editbot = edittop;
-	for (i = 0; (i <= editwinrows - 1) && (editbot->next != NULL)
-	     && (editbot->next != filebot); i++)
-	    editbot = editbot->next;
+	fix_editbot();
     }
 
-
     edit_refresh();
-    edit_refresh();		/* XXX FIXME XXX */
     statusbar("Justify Complete");
     return 1;
 #else
@@ -1534,24 +1509,25 @@ void help_init(void)
 
     /* Now add our shortcut info */
     for (i = 0; i < MAIN_LIST_LEN; i++) {
-	sofar = sprintf(buf, "^%c	", main_list[i].val + 64);
+	sofar = snprintf(buf, BUFSIZ, "^%c	", main_list[i].val + 64);
 
 	if (main_list[i].misc1 > KEY_F0 && main_list[i].misc1 <= KEY_F(64))
-	    sofar += sprintf(&buf[sofar], "(F%d)	",
+	    sofar += snprintf(&buf[sofar], BUFSIZ - sofar, "(F%d)	",
 			     main_list[i].misc1 - KEY_F0);
 	else
-	    sofar += sprintf(&buf[sofar], "	");
+	    sofar += snprintf(&buf[sofar], BUFSIZ - sofar, "	");
 
 	if (main_list[i].altval > 0)
-	    sofar += sprintf(&buf[sofar], "(@%c)	",
+	    sofar += snprintf(&buf[sofar], BUFSIZ - sofar, "(@%c)	",
 			     main_list[i].altval - 32);
 	else
-	    sofar += sprintf(&buf[sofar], "	");
+	    sofar += snprintf(&buf[sofar], BUFSIZ - sofar, "	");
 
 	if (main_list[i].help != NULL)
-	    sprintf(&buf[sofar], "%s\n", main_list[i].help);
+	    snprintf(&buf[sofar], BUFSIZ - sofar, "%s", main_list[i].help);
 
 	strcat(help_text, buf);
+	strcat(help_text, "\n");
     }
 
 }
@@ -1564,21 +1540,24 @@ int main(int argc, char *argv[])
     struct sigaction act;	/* For our lovely signals */
     int keyhandled = 0;		/* Have we handled the keystroke yet? */
     int tmpkey = 0, i;
-#ifdef HAVE_TABSIZE
-    int usrtabsize = 0;		/* User defined tab size */
-#endif
     char *argv0;
     struct termios term;
 
 #ifdef HAVE_GETOPT_LONG
     int option_index = 0;
     struct option long_options[] = {
+#ifdef _POSIX_VERSION
+        {"regexp", 0, 0, 'R'},
+#endif
 	{"version", 0, 0, 'V'},
 	{"const", 0, 0, 'c'},
 	{"suspend", 0, 0, 'z'},
 	{"nowrap", 0, 0, 'w'},
 	{"nohelp", 0, 0, 'x'},
 	{"help", 0, 0, 'h'},
+#ifndef NANO_SMALL
+	{"cut", 0, 0, 'k'},
+#endif
 	{"autoindent", 0, 0, 'i'},
 	{"tempfile", 0, 0, 't'},
 	{"speller", 1, 0, 's'},
@@ -1586,9 +1565,7 @@ int main(int argc, char *argv[])
 	{"mouse", 0, 0, 'm'},
 	{"pico", 0, 0, 'p'},
 	{"nofollow", 0, 0, 'l'},
-#ifdef HAVE_TABSIZE
-	{"tabsize", 0, 0, 'T'},
-#endif
+	{"tabsize", 1, 0, 'T'},
 	{0, 0, 0, 0}
     };
 #endif
@@ -1603,25 +1580,24 @@ int main(int argc, char *argv[])
 #endif
 
 #ifdef HAVE_GETOPT_LONG
-    while ((optchr = getopt_long(argc, argv, "?T:Vchilmpr:s:tvwxz",
+    while ((optchr = getopt_long(argc, argv, "?T:RVchiklmpr:s:tvwxz",
 				 long_options, &option_index)) != EOF) {
 #else
-    while ((optchr = getopt(argc, argv, "h?T:Vcilmpr:s:tvwxz")) != EOF) {
+    while ((optchr = getopt(argc, argv, "h?T:RVciklmpr:s:tvwxz")) != EOF) {
 #endif
 
 	switch (optchr) {
-#ifdef HAVE_TABSIZE
 	case 'T':
-	    usrtabsize = atoi(optarg);
-	    if (usrtabsize <= 0) {
+	    tabsize = atoi(optarg);
+	    if (tabsize <= 0) {
 		usage();	/* To stop bogus data for tab width */
 		finish(1);
 	    }
 	    break;
-#else
-	case 'T':
-	    usage();		/* Oops!  You dont really have that option */
-	    finish(1);
+#ifdef _POSIX_VERSION
+	case 'R':
+	    SET(USE_REGEXP);
+	    break;
 #endif
 	case 'V':
 	    version();
@@ -1636,6 +1612,15 @@ int main(int argc, char *argv[])
 	case 'i':
 	    SET(AUTOINDENT);
 	    break;
+#ifndef NANO_SMALL
+	case 'k':
+	    SET(CUT_TO_END);
+	    break;
+#else
+	case 'k':
+	    usage();		/* Oops!  You dont really have that option */
+	    finish(1);
+#endif
 	case 'l':
 	    UNSET(FOLLOW_SYMLINKS);
 	    break;
@@ -1657,7 +1642,7 @@ int main(int argc, char *argv[])
 	    strcpy(alt_speller, optarg);
 	    break;
 	case 't':
-	    temp_opt = 1;
+	    SET(TEMP_OPT);
 	    break;
 	case 'v':
 	    SET(VIEW_MODE);
@@ -1708,12 +1693,10 @@ int main(int argc, char *argv[])
     /* First back up the old settings so they can be restored, duh */
     tcgetattr(0, &oldterm);
 
-    /* Adam's code to blow away intr character so ^C can show cursor pos */
-    tcgetattr(0, &term);
-    for (i = 0; i < NCCS; i++) {
-	if (term.c_cc[i] == CINTR || term.c_cc[i] == CQUIT)
-	    term.c_cc[i] = 0;
-    }
+    term = oldterm;
+    term.c_cc[VINTR] = _POSIX_VDISABLE;
+    term.c_cc[VQUIT] = _POSIX_VDISABLE;
+    term.c_lflag &= ~IEXTEN;
     tcsetattr(0, TCSANOW, &term);
 
     /* now ncurses init stuff... */
@@ -1787,16 +1770,11 @@ int main(int argc, char *argv[])
     if (startline > 0)
 	do_gotoline(startline);
     else
-	edit_update(fileage);
+	edit_update(fileage, CENTER);
 
     edit_refresh();
     reset_cursor();
 
-#ifdef HAVE_TABSIZE
-    if (usrtabsize > 0)
-	TABSIZE = usrtabsize;
-#endif
-
     while (1) {
 	kbinput = wgetch(edit);
 	if (kbinput == 27) {	/* Grab Alt-key stuff first */
