@@ -1,3 +1,4 @@
+/* $Id: nano.h,v 1.10 2000/08/06 21:13:45 astyanax Exp $ */
 /**************************************************************************
  *   nano.h                                                               *
  *                                                                        *
@@ -44,11 +45,6 @@
 #include <curses.h> 
 #endif /* CURSES_H */
 
-#ifndef HAVE_TABSIZE
-    static int TABSIZE = 8;
-#endif
-
-
 #ifdef HAVE_LIBINTL_H
 #include <libintl.h>
 #endif
@@ -102,7 +98,10 @@ typedef struct shortcut {
 #define SAMELINEWRAP		(1<<11)
 #define VIEW_MODE		(1<<12)
 #define USE_MOUSE		(1<<13)
-
+#define USE_REGEXP              (1<<14)
+#define REGEXP_COMPILED         (1<<15)
+#define TEMP_OPT         	(1<<16)
+#define CUT_TO_END         	(1<<17)
 
 /* Control key sequences, chaning these would be very very bad */
 
@@ -184,7 +183,7 @@ know what you're doing */
 #define NANO_REPLACE_KEY	NANO_CONTROL_4
 #define NANO_REPLACE_FKEY	KEY_F(14)
 #define NANO_ALT_REPLACE_KEY	NANO_ALT_R
-#define NANO_OTHERSEARCH_KEY	NANO_CONTROL_T
+#define NANO_OTHERSEARCH_KEY	NANO_CONTROL_R
 #define NANO_PREVPAGE_KEY	NANO_CONTROL_Y
 #define NANO_PREVPAGE_FKEY	KEY_F(7)
 #define NANO_NEXTPAGE_KEY	NANO_CONTROL_V
@@ -216,11 +215,12 @@ know what you're doing */
 #define NANO_TAB_KEY		NANO_CONTROL_I
 #define NANO_SUSPEND_KEY	NANO_CONTROL_Z
 #define NANO_ENTER_KEY		NANO_CONTROL_M
+#define NANO_FROMSEARCHTOGOTO_KEY NANO_CONTROL_T
 
 #define MAIN_LIST_LEN 26
 #define MAIN_VISIBLE 12
-#define WHEREIS_LIST_LEN 5
-#define REPLACE_LIST_LEN 5
+#define WHEREIS_LIST_LEN 6
+#define REPLACE_LIST_LEN 6
 #define GOTO_LIST_LEN 3
 #define WRITEFILE_LIST_LEN 1
 #define HELP_LIST_LEN 3
@@ -229,4 +229,7 @@ know what you're doing */
 #define VIEW 1
 #define NOVIEW 0
 
+#define TOP 2
+#define CENTER 1
+#define BOTTOM 0
 #endif                             
