@@ -1,3 +1,4 @@
+/* $Id: search.c,v 1.16 2000/08/06 21:13:45 astyanax Exp $ */
 /**************************************************************************
  *   search.c                                                             *
  *                                                                        *
@@ -20,6 +21,7 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 #include <stdio.h>
 #include "config.h"
 #include "proto.h"
@@ -32,6 +34,22 @@
 #define _(string) (string)
 #endif
 
+/* Regular expression helper functions */
+
+#ifdef _POSIX_VERSION
+void regexp_init(const char *regexp)
+{
+    regcomp(&search_regexp, regexp, ISSET(CASE_SENSITIVE) ? 0 : REG_ICASE);
+    SET(REGEXP_COMPILED);
+}
+
+void regexp_cleanup()
+{
+    UNSET(REGEXP_COMPILED);
+    regfree(&search_regexp);
+}
+#endif
+
 /* Set up the system variables for a search or replace.  Returns -1 on
    abort, 0 on success, and 1 on rerun calling program 
    Return -2 to run opposite program (searchg -> replace, replace -> search)
@@ -41,18 +59,31 @@
 int search_init(int replacing)
 {
     int i;
-    char buf[135];
-
+    char buf[BUFSIZ];
+    char *prompt, *reprompt = "";
+ 
     if (last_search[0]) {
-	sprintf(buf, " [%s]", last_search);
+	snprintf(buf, BUFSIZ, " [%s]", last_search);
     } else {
 	buf[0] = '\0';
     }
 
+    if (ISSET(USE_REGEXP) && ISSET(CASE_SENSITIVE))
+        prompt = _("Case Sensitive Regexp Search%s%s");
+    else if (ISSET(USE_REGEXP))
+        prompt = _("Regexp Search%s%s");
+    else 
+    if (ISSET(CASE_SENSITIVE))
+        prompt = _("Case Sensitive Search%s%s");
+    else
+        prompt = _("Search%s%s");
+
+    if (replacing)
+	reprompt = _(" (to replace)");
+	        
     i = statusq(replacing ? replace_list : whereis_list,
 		replacing ? REPLACE_LIST_LEN : WHEREIS_LIST_LEN, "",
-		ISSET(CASE_SENSITIVE) ? _("Case Sensitive Search%s") :
-		_("Search%s"), buf);
+		prompt, reprompt, buf);
 
     /* Cancel any search, or just return with no previous search */
     if ((i == -1) || (i < 0 && !last_search[0])) {
@@ -61,9 +92,16 @@ int search_init(int replacing)
 	return -1;
     } else if (i == -2) {	/* Same string */
 	strncpy(answer, last_search, 132);
+#ifdef _POSIX_VERSION
+        if (ISSET(USE_REGEXP))
+            regexp_init(answer);
+#endif
     } else if (i == 0) {	/* They entered something new */
 	strncpy(last_search, answer, 132);
-
+#ifdef _POSIX_VERSION
+        if (ISSET(USE_REGEXP))
+            regexp_init(answer);
+#endif
 	/* Blow away last_replace because they entered a new search
 	   string....uh, right? =) */
 	last_replace[0] = '\0';
@@ -76,6 +114,9 @@ int search_init(int replacing)
 	return 1;
     } else if (i == NANO_OTHERSEARCH_KEY) {
 	return -2;		/* Call the opposite search function */
+    } else if (i == NANO_FROMSEARCHTOGOTO_KEY) {
+	do_gotoline_void();
+	return -3;
     } else {			/* First line key, etc. */
 	do_early_abort();
 	return -3;
@@ -88,6 +129,7 @@ filestruct *findnextstr(int quiet, filestruct * begin, char *needle)
 {
     filestruct *fileptr;
     char *searchstr, *found = NULL, *tmp;
+    int past_editbot = 0;
 
     fileptr = current;
 
@@ -95,8 +137,12 @@ filestruct *findnextstr(int quiet, filestruct * begin, char *needle)
     /* Look for searchstr until EOF */
     while (fileptr != NULL &&
 	   (found = strstrwrapper(searchstr, needle)) == NULL) {
+
 	fileptr = fileptr->next;
 
+	if (!past_editbot && (fileptr == editbot))
+	    past_editbot = 1;
+
 	if (fileptr == begin)
 	    return NULL;
 
@@ -111,7 +157,8 @@ filestruct *findnextstr(int quiet, filestruct * begin, char *needle)
 	for (tmp = fileptr->data; tmp != found; tmp++)
 	    current_x++;
 
-	edit_update(current);
+	if (past_editbot)
+	    edit_update(current, CENTER);
 	reset_cursor();
     } else {			/* We're at EOF, go back to the top, once */
 
@@ -133,7 +180,7 @@ filestruct *findnextstr(int quiet, filestruct * begin, char *needle)
 	    for (tmp = fileptr->data; tmp != found; tmp++)
 		current_x++;
 
-	    edit_update(current);
+	    edit_update(current, CENTER);
 	    reset_cursor();
 
 	    if (!quiet)
@@ -154,6 +201,11 @@ void search_abort(void)
     UNSET(KEEP_CUTBUFFER);
     display_main_list();
     wrefresh(bottomwin);
+
+#ifdef _POSIX_VERSION
+    if (ISSET(REGEXP_COMPILED))
+        regexp_cleanup();
+#endif
 }
 
 /* Search for a string */
@@ -171,7 +223,6 @@ int do_search(void)
 	search_abort();
 	return 0;
     } else if (i == -2) {
-	search_abort();
 	do_replace();
 	return 0;
     } else if (i == 1) {
@@ -194,17 +245,134 @@ void print_replaced(int num)
 
 void replace_abort(void)
 {
-    UNSET(KEEP_CUTBUFFER);
-    display_main_list();
-    reset_cursor();
+    /* Identicle to search_abort, so we'll call it here.  If it
+       does something different later, we can change it back.  For now
+       it's just a waste to duplicat code */
+    search_abort();
 }
 
-/* Search for a string */
+#ifdef _POSIX_VERSION
+int replace_regexp(char *string, int create_flag)
+{
+    /* split personality here - if create_flag is null, just calculate
+     * the size of the replacement line (necessary because of
+     * subexpressions like \1 \2 \3 in the replaced text) */
+
+    char *c;
+    int new_size = strlen(current->data) + 1;
+    int search_match_count = regmatches[0].rm_eo -
+        regmatches[0].rm_so;
+
+    new_size -= search_match_count;
+
+    /* Iterate through the replacement text to handle
+     * subexpression replacement using \1, \2, \3, etc */
+
+    c = last_replace;
+    while (*c) {
+        if (*c != '\\') {
+            if (create_flag)
+                *string++=*c;
+            c++;
+            new_size++;
+        } else {
+            int num = (int)*(c+1) - (int)'0';
+            if (num >= 1 && num <= 9) {
+
+                int i = regmatches[num].rm_so;
+
+                if (num > search_regexp.re_nsub) {
+                    /* Ugh, they specified a subexpression that doesn't
+                       exist.  */
+                    return -1;
+                }
+
+                /* Skip over the replacement expression */
+                c+=2;
+
+                /* But add the length of the subexpression to new_size */
+                new_size += regmatches[num].rm_eo - regmatches[num].rm_so;
+
+                /* And if create_flag is set, append the result of the
+                 * subexpression match to the new line */
+                while (create_flag && i < regmatches[num].rm_eo )
+                    *string++=*(current->data + i++);
+               
+            } else {
+                if (create_flag)
+                    *string++=*c;
+                c++;
+                new_size++;
+            }
+        }
+    }
+
+    if (create_flag)
+        *string = 0;
+
+    return new_size;
+}
+#endif
+    
+char *replace_line()
+{
+    char *copy, *tmp;
+    int new_line_size;
+    int search_match_count;
+
+    /* Calculate size of new line */
+#ifdef _POSIX_VERSION
+    if (ISSET(USE_REGEXP)) {
+        search_match_count = regmatches[0].rm_eo -
+            regmatches[0].rm_so;
+        new_line_size = replace_regexp(NULL, 0);
+        /* If they specified an invalid subexpression in the replace
+         * text, return NULL indicating an error */
+        if (new_line_size < 0)
+            return NULL;
+    } else {
+#else
+    {
+#endif
+        search_match_count = strlen(last_search);
+        new_line_size = strlen(current->data) - strlen(last_search) +
+            strlen(last_replace) + 1;
+    }
+    
+    /* Create buffer */
+    copy = nmalloc(new_line_size);
+
+    /* Head of Original Line */
+    strncpy(copy, current->data, current_x);
+    copy[current_x] = 0;
+
+    /* Replacement Text */
+    if (!ISSET(USE_REGEXP))
+        strcat(copy, last_replace);
+#ifdef _POSIX_VERSION
+    else
+        (void)replace_regexp(copy + current_x, 1);
+#endif
+
+    /* The tail of the original line */
+    /* This may expose other bugs, because it no longer
+       goes through each character on the string
+       and tests for string goodness.  But because
+       we can assume the invariant that current->data
+       is less than current_x + strlen(last_search) long,
+       this should be safe.  Or it will expose bugs ;-) */
+    tmp = current->data + current_x + search_match_count;
+    strcat(copy, tmp);
+
+    return copy;
+}
+
+/* Replace a string */
 int do_replace(void)
 {
     int i, replaceall = 0, numreplaced = 0, beginx;
     filestruct *fileptr, *begin;
-    char *tmp, *copy, prevanswer[132] = "";
+    char *copy, prevanswer[132] = "";
 
     if ((i = search_init(1)) == -1) {
 	statusbar(_("Replace Cancelled"));
@@ -214,7 +382,6 @@ int do_replace(void)
 	do_replace();
 	return 1;
     } else if (i == -2) {
-	replace_abort();
 	do_search();
 	return 0;
     } else if (i == -3) {
@@ -242,7 +409,7 @@ int do_replace(void)
 
 	    do_replace();
 	    return 0;
-	} else {		/* First page, last page, for example could get here */
+	} else if (i != -2 ) {		/* First page, last page, for example could get here */
 
 	    do_early_abort();
 	    replace_abort();
@@ -297,33 +464,19 @@ int do_replace(void)
 	    if (i == 2)
 		replaceall = 1;
 
-	    /* Create buffer */
-	    copy = nmalloc(strlen(current->data) - strlen(last_search) +
-			   strlen(last_replace) + 1);
-
-	    /* Head of Original Line */
-	    strncpy(copy, current->data, current_x);
-	    copy[current_x] = 0;
-
-	    /* Replacement Text */
-	    strcat(copy, last_replace);
-
-	    /* The tail of the original line */
-	    /*  This may expose other bugs, because it no longer
-	       goes through each character on the string
-	       and tests for string goodness.  But because
-	       we can assume the invariant that current->data
-	       is less than current_x + strlen(last_search) long,   
-	       this should be safe.  Or it will expose bugs ;-) */
-	    tmp = current->data + current_x + strlen(last_search);
-	    strcat(copy, tmp);
+            copy = replace_line();
+            if (!copy) {
+                statusbar("Replace failed: unknown subexpression!");
+                replace_abort();
+		return 0;
+            }
 
 	    /* Cleanup */
 	    free(current->data);
 	    current->data = copy;
 
 	    /* Stop bug where we replace a substring of the replacement text */
-	    current_x += strlen(last_replace);
+	    current_x += strlen(last_replace) - 1;
 
 	    edit_refresh();
 	    set_modified();
@@ -335,7 +488,7 @@ int do_replace(void)
     current = begin;
     current_x = beginx;
     renumber_all();
-    edit_update(current);
+    edit_update(current, CENTER);
     print_replaced(numreplaced);
     replace_abort();
     return 1;
@@ -369,7 +522,7 @@ int do_gotoline(long defline)
 	if (!strcmp(answer, "$")) {
 	    current = filebot;
 	    current_x = 0;
-	    edit_update(current);
+	    edit_update(current, CENTER);
 	    goto_abort();
 	    return 1;
 	}
@@ -387,14 +540,14 @@ int do_gotoline(long defline)
 		  filebot->lineno);
 	current = filebot;
 	current_x = 0;
-	edit_update(current);
+	edit_update(current, CENTER);
     } else {
 	for (fileptr = fileage; fileptr != NULL && i < line; i++)
 	    fileptr = fileptr->next;
 
 	current = fileptr;
 	current_x = 0;
-	edit_update(current);
+	edit_update(current, CENTER);
     }
 
     goto_abort();
