@@ -1,3 +1,4 @@
+/* $Id: utils.c,v 1.7 2000/08/06 21:13:45 astyanax Exp $ */
 /**************************************************************************
  *   utils.c                                                              *
  *                                                                        *
@@ -18,6 +19,7 @@
  *                                                                        *
  **************************************************************************/
 
+#include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
@@ -80,7 +82,14 @@ char *strcasestr(char *haystack, char *needle)
 
 char *strstrwrapper(char *haystack, char *needle)
 {
-
+#ifdef _POSIX_VERSION
+    if (ISSET(USE_REGEXP)) {
+      int result=regexec(&search_regexp, haystack, 10, regmatches, 0);
+      if (!result)
+          return haystack+regmatches[0].rm_so;
+      return 0;
+    }  
+#endif
     if (ISSET(CASE_SENSITIVE))
 	return strstr(haystack, needle);
     else
@@ -109,3 +118,15 @@ void *nrealloc(void *ptr, size_t howmuch)
 
     return r;
 }
+
+/* Append a new magic-line to filebot */
+void new_magicline(void) {
+    filebot->next = nmalloc(sizeof(filestruct));
+    filebot->next->data = nmalloc(1);
+    filebot->next->data[0] = '\0';
+    filebot->next->prev = filebot;
+    filebot->next->next = NULL;
+    filebot->next->lineno = filebot->lineno + 1;
+    filebot = filebot->next;
+    totlines++;
+}
