@@ -1,3 +1,4 @@
+/* $Id: winio.c,v 1.29 2000/08/06 21:13:45 astyanax Exp $ */
 /**************************************************************************
  *   winio.c                                                              *
  *                                                                        *
@@ -20,6 +21,7 @@
 
 #include <stdarg.h>
 #include <string.h>
+#include <stdlib.h>
 #include "config.h"
 #include "proto.h"
 #include "nano.h"
@@ -47,7 +49,7 @@ int do_first_line(void)
     current = fileage;
     placewewant = 0;
     current_x = 0;
-    edit_update(current);
+    edit_update(current, CENTER);
     return 1;
 }
 
@@ -56,7 +58,7 @@ int do_last_line(void)
     current = filebot;
     placewewant = 0;
     current_x = 0;
-    edit_update(current);
+    edit_update(current, CENTER);
     return 1;
 }
 
@@ -72,9 +74,9 @@ int xpt(filestruct * fileptr, int index)
 	tabs++;
 
 	if (fileptr->data[i] == NANO_CONTROL_I) {
-	    if (tabs % TABSIZE == 0);
+	    if (tabs % tabsize == 0);
 	    else
-		tabs += TABSIZE - (tabs % TABSIZE);
+		tabs += tabsize - (tabs % tabsize);
 	} else if (fileptr->data[i] & 0x80)
 	    /* Make 8 bit chars only 1 collumn! */
 	    ;
@@ -105,10 +107,10 @@ int actual_x_from_start(filestruct * fileptr, int xplus, int start)
 
     for (i = start; tot <= xplus && fileptr->data[i] != 0; i++, tot++)
 	if (fileptr->data[i] == NANO_CONTROL_I) {
-	    if (tot % TABSIZE == 0)
+	    if (tot % tabsize == 0)
 		tot++;
 	    else
-		tot += TABSIZE - (tot % TABSIZE);
+		tot += tabsize - (tot % tabsize);
 	} else if (fileptr->data[i] & 0x80)
 	    tot++;		/* Make 8 bit chars only 1 column (again) */
 	else if (fileptr->data[i] < 32)
@@ -139,9 +141,9 @@ int strlenpt(char *buf)
 	tabs++;
 
 	if (buf[i] == NANO_CONTROL_I) {
-	    if (tabs % TABSIZE == 0);
+	    if (tabs % tabsize == 0);
 	    else
-		tabs += TABSIZE - (tabs % TABSIZE);
+		tabs += tabsize - (tabs % tabsize);
 	} else if (buf[i] & 0x80)
 	    /* Make 8 bit chars only 1 collumn! */
 	    ;
@@ -237,14 +239,10 @@ int nanogetstr(char *buf, char *def, shortcut s[], int slen, int start_x)
 	strcpy(answer, def);
 	strcpy(inputbuf, def);
     }
-    /* Go into raw mode so we can actually get ^C, for example */
-    raw();
 
     while ((kbinput = wgetch(bottomwin)) != 13) {
 	for (j = 0; j <= slen - 1; j++) {
 	    if (kbinput == s[j].val) {
-		noraw();
-		cbreak();
 		strcpy(answer, "");
 		return s[j].val;
 	    }
@@ -417,8 +415,6 @@ int nanogetstr(char *buf, char *def, shortcut s[], int slen, int start_x)
 
     strncpy(answer, inputbuf, 132);
 
-    noraw();
-    cbreak();
     if (!strcmp(answer, ""))
 	return -2;
     else
@@ -480,7 +476,6 @@ void clear_bottomwin(void)
 
     mvwaddstr(bottomwin, 1, 0, hblank);
     mvwaddstr(bottomwin, 2, 0, hblank);
-    wrefresh(bottomwin);
 }
 
 void bottombars(shortcut s[], int slen)
@@ -497,7 +492,7 @@ void bottombars(shortcut s[], int slen)
     clear_bottomwin();
     wmove(bottomwin, 1, 0);
     for (i = 0; i <= slen - 1; i += 2) {
-	sprintf(keystr, "^%c", s[i].val + 64);
+	snprintf(keystr, 10, "^%c", s[i].val + 64);
 	onekey(keystr, s[i].desc);
 
 	for (j = 0; j < k; j++)
@@ -506,7 +501,7 @@ void bottombars(shortcut s[], int slen)
 
     wmove(bottomwin, 2, 0);
     for (i = 1; i <= slen - 1; i += 2) {
-	sprintf(keystr, "^%c", s[i].val + 64);
+	snprintf(keystr, 10, "^%c", s[i].val + 64);
 	onekey(keystr, s[i].desc);
 
 	for (j = 0; j < k; j++)
@@ -527,6 +522,10 @@ void set_modified(void)
     }
 }
 
+/* And so start the display update routines */
+/* Given a column, this returns the "page" it is on  */
+/* "page" in the case of the display columns, means which set of 80 */
+/* characters is viewable (ie: page 1 shows from 1 to COLS) */
 inline int get_page_from_virtual(int virtual)
 {
     int page = 2;
@@ -543,6 +542,7 @@ inline int get_page_from_virtual(int virtual)
     return page;
 }
 
+/* The inverse of the above function */
 inline int get_page_start_virtual(int page)
 {
     int virtual;
@@ -558,16 +558,25 @@ inline int get_page_end_virtual(int page)
 }
 
 #ifndef NANO_SMALL
-/* Called only from edit_add, and therefore expects a
- * converted-to-only-spaces line */
+/* This takes care of the case where there is a mark that covers only */
+/* the current line. */
+
+/* It expects a line with no tab characers (ie: the type that edit_add */
+/* deals with */
 void add_marked_sameline(int begin, int end, filestruct * fileptr, int y,
 			 int virt_cur_x, int this_page)
 {
+    /*
+     * The general idea is to break the line up into 3 sections: before
+     * the mark, the mark, and after the mark.  We then paint each in
+     * turn (for those that are currently visible, of course
+     *
+     * 3 start points: 0 -> begin, begin->end, end->strlen(data)
+     *    in data    :    pre          sel           post        
+     */
     int this_page_start = get_page_start_virtual(this_page),
 	this_page_end = get_page_end_virtual(this_page);
 
-    /* 3 start points: 0 -> begin, begin->end, end->strlen(data) */
-    /*    in data    :    pre          sel           post        */
     /* likewise, 3 data lengths */
     int pre_data_len = begin, sel_data_len = end - begin, post_data_len = 0;	/* Determined from the other two */
 
@@ -584,17 +593,17 @@ void add_marked_sameline(int begin, int end, filestruct * fileptr, int y,
     if (end < this_page_start)
 	end = this_page_start;
 
-    /* we don't care about end, because it will just get cropped
-     * due to length */
     if (begin > this_page_end)
 	begin = this_page_end;
 
     if (end > this_page_end)
 	end = this_page_end;
 
+    /* Now calculate the lengths */
     sel_data_len = end - begin;
     post_data_len = this_page_end - end;
 
+    /* Paint this line! */
     mvwaddnstr(edit, y, 0, &fileptr->data[this_page_start], pre_data_len);
     wattron(edit, A_REVERSE);
     mvwaddnstr(edit, y, begin - this_page_start,
@@ -605,39 +614,57 @@ void add_marked_sameline(int begin, int end, filestruct * fileptr, int y,
 }
 #endif
 
-/* Called only from update_line.  Expects a converted-to-not-have-tabs
+/* edit_add takes care of the job of actually painting a line into the
+ * edit window.
+ * 
+ * Called only from update_line.  Expects a converted-to-not-have-tabs
  * line */
 void edit_add(filestruct * fileptr, int yval, int start, int virt_cur_x,
 	      int virt_mark_beginx, int this_page)
 {
 #ifndef NANO_SMALL
-    if (ISSET(MARK_ISSET)) {
-	if ((fileptr->lineno > mark_beginbuf->lineno
+    /* There are quite a few cases that could take place, we'll deal
+     * with them each in turn */
+    if (ISSET(MARK_ISSET)    
+	&& !((fileptr->lineno > mark_beginbuf->lineno
 	     && fileptr->lineno > current->lineno)
 	    || (fileptr->lineno < mark_beginbuf->lineno
-		&& fileptr->lineno < current->lineno)) {
-
-	    /* We're on a normal, unselected line */
-	    mvwaddnstr(edit, yval, 0, fileptr->data, COLS);
-	} else {
+		&& fileptr->lineno < current->lineno)))
+    {
+	    /* If we get here we are on a line that is atleast
+	     * partially selected.  The lineno checks above determined
+	     * that */
 	    if (fileptr != mark_beginbuf && fileptr != current) {
-		/* We're on selected text */
+		/* We are on a completely marked line, paint it all
+		 * inverse */
 		wattron(edit, A_REVERSE);
 		mvwaddnstr(edit, yval, 0, fileptr->data, COLS);
 		wattroff(edit, A_REVERSE);
 	    } else if (fileptr == mark_beginbuf && fileptr == current) {
 		/* Special case, we're still on the same line we started
-		 * marking */
-		if (virt_cur_x < virt_mark_beginx)
+		 * marking -- so we call our helper function */
+		if (virt_cur_x < virt_mark_beginx) {
+		    /* To the right of us is marked */
 		    add_marked_sameline(virt_cur_x, virt_mark_beginx,
 					fileptr, yval, virt_cur_x,
 					this_page);
-		else
+		} else {
+		    /* To the left of us is marked */
 		    add_marked_sameline(virt_mark_beginx, virt_cur_x,
 					fileptr, yval, virt_cur_x,
 					this_page);
+		}
 	    } else if (fileptr == mark_beginbuf) {
-		/* we're updating the line that was first marked */
+		/*
+		 * we're updating the line that was first marked
+		 * but we're not currently on it.  So we want to
+		 * figur out which half to invert based on our
+		 * relative line numbers.
+		 *
+		 * i.e. If we're above the "beginbuf" line, we want to
+		 * mark the left side.  Otherwise we're below, so we
+		 * mark the right
+		 */
 		int target;
 
 		if (mark_beginbuf->lineno > current->lineno)
@@ -666,7 +693,7 @@ void edit_add(filestruct * fileptr, int yval, int start, int virt_cur_x,
 
 	    } else if (fileptr == current) {
 		/* we're on the cursors line, but it's not the first
-		 * one we marked... */
+		 * one we marked.  Similar to the previous logic. */
 		int this_page_start = get_page_start_virtual(this_page),
 		    this_page_end = get_page_end_virtual(this_page);
 
@@ -697,17 +724,18 @@ void edit_add(filestruct * fileptr, int yval, int start, int virt_cur_x,
 
 		if (mark_beginbuf->lineno > current->lineno)
 		    wattroff(edit, A_REVERSE);
-	    }
 	}
 
     } else
 #endif
+    /* Just paint the string (no mark on this line) */
 	mvwaddnstr(edit, yval, 0, &fileptr->data[start],
 		   get_page_end_virtual(this_page) - start);
 }
 
 /*
- * Just update one line in the edit buffer 
+ * Just update one line in the edit buffer.  Basically a wrapper for
+ * edit_add
  *
  * index gives is a place in the string to update starting from.
  * Likely args are current_x or 0.
@@ -746,7 +774,7 @@ void update_line(filestruct * fileptr, int index)
 		    virt_cur_x++;
 		if (i < mark_beginx)
 		    virt_mark_beginx++;
-	    } while (pos % TABSIZE);
+	    } while (pos % tabsize);
 	    /* must decrement once to account for tab-is-one-character */
 	    if (i < current_x)
 		virt_cur_x--;
@@ -795,7 +823,8 @@ void center_cursor(void)
 /* Refresh the screen without changing the position of lines */
 void edit_refresh(void)
 {
-    int lines = 0, i = 0;
+    static int noloop = 0;
+    int lines = 0, i = 0, currentcheck = 0;
     filestruct *temp, *hold = current;
 
     if (current == NULL)
@@ -806,9 +835,21 @@ void edit_refresh(void)
     while (lines <= editwinrows - 1 && lines <= totlines && temp != NULL) {
 	hold = temp;
 	update_line(temp, current_x);
+	if (temp == current)
+	    currentcheck = 1;
+
 	temp = temp->next;
 	lines++;
     }
+    /* If noloop == 1, then we already did an edit_update without finishing
+       this function.  So we don't run edit_update again */
+    if (!currentcheck && !noloop) /* Then current has run off the screen... */ 
+    {
+	edit_update(current, CENTER);
+	noloop = 1;
+    }
+    else if (noloop)
+	noloop = 0;
 
     if (lines <= editwinrows - 1)
 	while (lines <= editwinrows - 1) {
@@ -825,63 +866,30 @@ void edit_refresh(void)
  * Nice generic routine to update the edit buffer given a pointer to the
  * file struct =) 
  */
-void edit_update(filestruct * fileptr)
+void edit_update(filestruct * fileptr, int topmidbot)
 {
-
-    int lines = 0, i = 0;
+    int i = 0;
     filestruct *temp;
 
     if (fileptr == NULL)
 	return;
 
     temp = fileptr;
-    while (i <= editwinrows / 2 && temp->prev != NULL) {
-	i++;
-	temp = temp->prev;
-    }
-    edittop = temp;
+    if (topmidbot == 2)
+	;
+    else if (topmidbot == 0)
+	for (i = 0; i <= editwinrows - 1 && temp->prev != NULL; i++)
+	    temp = temp->prev;
+    else
+	for (i = 0; i <= editwinrows / 2 && temp->prev != NULL; i++)
+	    temp = temp->prev;
 
-    while (lines <= editwinrows - 1 && lines <= totlines && temp != NULL
-	   && temp != filebot) {
-	temp = temp->next;
-	lines++;
-    }
-    editbot = temp;
+    edittop = temp;
+    fix_editbot();
 
     edit_refresh();
 }
 
-/* Now we want to update the screen using this struct as the top of the edit buffer */
-void edit_update_top(filestruct * fileptr)
-{
-    int i;
-    filestruct *temp = fileptr;
-
-    if (fileptr->next == NULL || fileptr == NULL)
-	return;
-
-    i = 0;
-    while (i <= editwinrows / 2 && temp->next != NULL) {
-	i++;
-	temp = temp->next;
-    }
-    edit_update(temp);
-}
-
-/* And also for the bottom... */
-void edit_update_bot(filestruct * fileptr)
-{
-    int i;
-    filestruct *temp = fileptr;
-
-    i = 0;
-    while (i <= editwinrows / 2 - 1 && temp->prev != NULL) {
-	i++;
-	temp = temp->prev;
-    }
-    edit_update(temp);
-}
-
 /* This function updates current based on where current_y is, reset_cursor 
    does the opposite */
 void update_cursor(void)
@@ -921,8 +929,8 @@ int statusq(shortcut s[], int slen, char *def, char *msg, ...)
 
     va_start(ap, msg);
     vsnprintf(foo, 132, msg, ap);
-    strncat(foo, ": ", 132);
     va_end(ap);
+    strncat(foo, ": ", 132);
 
     wattron(bottomwin, A_REVERSE);
     ret = nanogetstr(foo, def, s, slen, (strlen(foo) + 3));
@@ -965,14 +973,15 @@ int do_yesno(int all, int leavecursor, char *msg, ...)
     blank_statusbar_refresh();
     wattroff(bottomwin, A_REVERSE);
 
+    /* Remove gettext call for keybindings until we clear the thing up */
     if (!ISSET(NO_HELP)) {
 	wmove(bottomwin, 1, 0);
-	onekey(_(" Y"), _("Yes"));
+	onekey(" Y", _("Yes"));
 	if (all)
-	    onekey(_(" A"), _("All"));
+	    onekey(" A", _("All"));
 	wmove(bottomwin, 2, 0);
-	onekey(_(" N"), _("No"));
-	onekey(_("^C"), _("Cancel"));
+	onekey(" N", _("No"));
+	onekey("^C", _("Cancel"));
     }
     va_start(ap, msg);
     vsnprintf(foo, 132, msg, ap);
@@ -985,8 +994,6 @@ int do_yesno(int all, int leavecursor, char *msg, ...)
     if (leavecursor == 1)
 	reset_cursor();
 
-    raw();
-
     while (ok == -1) {
 	kbinput = wgetch(edit);
 
@@ -1009,8 +1016,6 @@ int do_yesno(int all, int leavecursor, char *msg, ...)
 	    break;
 	}
     }
-    noraw();
-    cbreak();
 
     /* Then blank the screen */
     blank_statusbar_refresh();
@@ -1267,3 +1272,12 @@ void dump_buffer_reverse(filestruct * inptr)
     }
 #endif				/* DEBUG */
 }
+
+/* Fix editbot based on the assumption that edittop is correct */
+void fix_editbot(void) {
+    int i;
+    editbot = edittop;
+    for(i = 0; (i <= editwinrows - 1) && (editbot->next != NULL)
+	&& (editbot != filebot); i++, editbot = editbot->next);
+}
+
